<?php  

class db {

	private static $db;
	protected $table;
	protected $properties;
	protected $positions;
	protected $unserializekeys = false;
	protected $fields = array('1', '2', '3', '4');
	public $id;
	public $lastId = false;
	public $status;
	public $msg;
	public $result;
	public $user;
	public $changed = false;
	
	function __construct(){
		date_default_timezone_set('CET');
		$host = DB_HOST;
		$port = DB_PORT;
		$name = DB_NAME;
		$username = DB_USER;
		$password = DB_PASSWORD;
		try {
			self::$db = new PDO("mysql:host=$host;port=$port;dbname=$name",$username,$password);
			self::$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			$this->status = true;
		} catch(PDOException $e) {
			$this->msg = $e->getMessage();
			$this->status = false;
		}
	}
	
	protected function databaseConnection(){
		if (is_object(self::$db)){
			return true;
		} else {
			return false;
		}
	}
	
	protected function fetchSql($sql="",$vals=null) {
		if (is_object(self::$db)){
			try {
				$statement = self::$db->prepare($sql);
				$statement->execute($vals);
				$rows = $statement->fetchALL(PDO::FETCH_OBJ);
				$result = (count($rows) > 0) ? $rows : false;
				$this->status = ($rows) ? true : false;
				return $result;
			} catch(PDOException $e) {
				$this->msg = $e->getMessage();
				return $this->status = false;
			}
		}
	}	

	protected function create(){
		if (is_object(self::$db)){
			$properties = $this->properties;
			$properties['last_changed'] = date("Y-m-d H:i:s");
			$sql = "INSERT INTO " . $this->table . " (";
			$sql .= join(", ", array_keys($properties));
			$sql .= ") VALUES (";
			$c = count($properties); $i = 1;
			foreach ($properties as $value){
				$sql .= ($i < $c)? self::$db->quote($value).', ' : self::$db->quote($value);
				$i++;
			}
			$sql .= ")";
			$statement = self::$db->prepare($sql);
			try {
				$statement->execute();
				$this->lastId = self::$db->lastInsertId();
				$this->status = true;
			} catch(PDOException $e) {
				$this->msg = $e->getMessage();
				$this->status = false;
			}
			return $this->status;
		}
	}
	
	protected function update($whereclause = false, $update=true){
		if (is_object(self::$db)){
			$properties = $this->properties;
			$property_pairs = array();
			foreach($properties as $key => $value){
				$property_pairs[] = "{$key}=".self::$db->quote($value);
			}
			$sql = "UPDATE " . $this->table . " SET ";
			$sql .= join(", ", $property_pairs);
			if ($update){
				$sql .= ", last_changed='".date("Y-m-d H:i:s")."' ";
			}
			if ($whereclause){
				$sql .= " WHERE ";
				if (is_array($whereclause)){
					$whereclauselenght = count($whereclause);
					$i = 1;
					foreach ($whereclause as $field=>$value){
						$sql .= $field." = '".$value."' ";
						if ($i < $whereclauselenght){
							$sql .= "AND ";
						}
						$i++;
					}
				} else {
					$sql .= $whereclause;
				}
			} else {
				$sql .= " WHERE id = " . $this->id;
			}
			$statement = self::$db->prepare($sql);
			try {
				$statement->execute();
				$this->status = true;
			} catch(PDOException $e) {
				$this->msg = $e->getMessage();
				$this->status = false;
			}
			return $this->status;
		}
	}
	
	protected function updatePosition(){
		if (is_object(self::$db)){
			if (is_array($this->positions)){
				$stmt = self::$db->prepare("UPDATE $this->table SET sort=:pos , field=:field WHERE id=:id");
	    		$stmt->bindParam(':id', $id, PDO::PARAM_INT);
	    		$stmt->bindParam(':pos', $pos, PDO::PARAM_INT);
	    		$stmt->bindParam(':field', $this->field, PDO::PARAM_INT);
	    		foreach ($this->positions as $pos => $id){
	    			try {
						$stmt->execute();
						$this->status = true;
					} catch(PDOException $e) {
						$this->msg = $e->getMessage();
						$this->status = false;
					}
	    		}
			} else {
				$this->status = false;
			}
			return $this->status;
		}
	}	
	
	protected function dbDelete($whereclause = false){
		if (is_object(self::$db)){
			$sql = "DELETE FROM " . $this->table;
			if ($whereclause){
				$sql .= " WHERE ";
				if (is_array($whereclause)){
					$whereclauselenght = count($whereclause);
					$i = 1;
					foreach ($whereclause as $field=>$value){
						$sql .= $field." = '".$value."' ";
						if ($i < $whereclauselenght){
							$sql .= "AND ";
						}
						$i++;
					}
				} else {
					$sql .= $whereclause;
				}
			} else {
				$sql .= " WHERE id = " . $this->id;
			}
			$statement = self::$db->prepare($sql);
			try {
				$statement->execute();
				$this->status = true;
			} catch(PDOException $e) {
				$this->msg = $e->getMessage();
				$this->status = false;
			}
			return $this->status;
		}
	}
	
	protected function serializeProperties() {
		foreach ($this->tableFields as $field){
			if (isset($_POST[$field])){
				if ($field == 'id'){
					$this->id = $_POST[$field];
				} else if (is_array($_POST[$field])){
					$this->properties[$field] = ($serialized = serialize($_POST[$field])) ? $serialized : utf8_decode($_POST[$field]);
				} else {
					$this->properties[$field] = utf8_decode($_POST[$field]);
				}
			} else if ( $field == 'path' && $this->files != false ) {
				$this->properties[$field] = $this->upload;
			} else if ( $field == 'thumb' && $this->files != false ) {
				$this->properties[$field] = $this->thumbnail;
			} else if ( $field == 'html' && $this->files != false ) {
				if (is_array($this->html)){
					$this->properties[$field] = serialize($this->html);
				} else {
					$this->properties[$field] = $this->html;
				}
			}
		}
	}
	
	protected function serializePositions() {
		if (is_array($_POST['sort'])){
			$this->positions = $_POST['sort'];
		}
	}
	
	protected function serializeValue($value) {
		if (isset($_POST[$value])){
			if ($field == 'id'){
				$this->id = $_POST[$field];
			} else if (is_array($_POST[$value])){
				$this->properties[$value] = (serialize($_POST[$value])) ? serialize($_POST[$value]) : $_POST[$value];
			} else {
				$this->properties[$value] = $_POST[$value];
			}
			return true;
		} else {
			return false;
		}
	}
		
	protected function unserializeResult($obj){
		if (is_array($obj) && $this->unserializekeys){
			foreach ($obj as $key=>$keys){
				if ($keys){
					foreach ($keys as $dbkey=>$content){
						if (in_array($dbkey, $this->unserializekeys) && $unserialized = unserialize($content)){
							$obj[$key]->$dbkey = $unserialized;
						} else {
							$obj[$key]->$dbkey = stripslashes(utf8_encode($content));
						}
					}
				}
			}
		}
		return $obj;
	}

	public function jsonResponse() {
		$json = getPublicObjectVars($this);
		foreach ($json as $key=>$val){
			if ($key == 'lang' || is_object($val)){
				unset($json[$key]);
			} else {
				if (!is_bool($val)){
					$json[$key] = utf8_encode($val);
				}
			}
		}
		echo json_encode($json);
	}

}

?>